from ash import *
import csv

num_cores = 16

pdb_nam = "protein_and_ligand.pdb"
frag_prot = Fragment(pdbfile = pdb_nam)

# This 
thisdir = "/home/jzw5/quantum-olfaction-data/2024-05_protlig/2024-06_pyscf"

# For now, we're going to try only the ligand as active site.
lig_ind = [4836, 4837, 4838, 4839, 4840, 4841, 4842, 4843, 4844, 4845]
#         [C1,   C2,   C3,   O2,   O2,   H21,  H22,  H31,  H32,  H33]
lig_mass_h5 = [12.01, 12.01, 12.01, 16.00, 16.00,
               1.008, 1.008, 1.008, 1.008, 1.008]
lig_mass_d5 = [12.01, 12.01, 12.01, 16.00, 16.00,
               2.014, 2.014, 2.014, 2.014, 2.014]

# MM region from OpenMM
# N.B.: I believe CHARMM36 actually means CHARMM36m, see
#   https://github.com/openmm/openmmforcefields/issues/298
# (CHARMM36m is an improved version, see
#   https://doi.org/10.1038/nmeth.4067)
# Creating the topology was a journey; see the associated .md file.
mm = OpenMMTheory(pdbfile = pdb_nam, 
                  GROMACSfiles = True,
                  grofile = "protein_and_ligand.gro",
                  gromacstopdir = thisdir,
                  gromacstopfile = "topol.top",
                  periodic = True,
                  platform = 'CPU',
                  numcores = num_cores,
                  autoconstraints = None,
                  rigidwater = False)

# QM region from ORCA (automatically generated RI: AutoAux)
#qm = ORCATheory(orcasimpleinput = "! B3LYP D4 ma-def2-TZVP AutoAux",
#                numcores = num_cores)

# QM region with PySCF.
# Basis:      minimally augmented Karlsruhe (doi:10.1007/s00214-010-0846-z)
#             we'll start with def2-TZVP, can *-TZVPP, *-QZVPP if needed
# Functional: B3LYP (reasonable accuracy, efficient, analytic gradient)
#             with D4 dispersion correciton
qm = PySCFTheory(numcores = num_cores,
                 scf_type = 'RKS',
                 basis = 'aug-cc-pVTZ',
                 functional = 'b3lyp',
                 dispersion = 'D4')

# QM/MM object combines
qmmm = QMMMTheory(qm_theory = qm, mm_theory = mm,
                  fragment = frag_prot,
                  qm_charge = -1,
                  embedding = "Elstat",
                  qmatoms = lig_ind,
                  printlevel = 2)

# QM/MM geometry optimization only of the ligand!
active_ind = lig_ind
geom_opt = Optimizer(fragment = frag_prot, theory = qmmm,
                     ActiveRegion = True, actatoms = active_ind,
                     maxiter = 200,
                     charge = -1, mult = 1)

print("Optimized geometry! ", geom_opt)

# Vibrational frequency calculation.
#   hessatoms = [active_ind] is a partial Hessian for just the active site!
#   hessatoms_masses = [] specifies the isotopes. Should be length of hessatoms

# Various sub-objects of interest:
#   num_freq.hessian; 
#   num_freq.frequencies (cm**-1); 
#   num_freq.IR_intensities (km/mol)
freq_h5 = NumFreq(fragment = frag_prot, theory = qmmm,
                  hessatoms = lig_ind, hessatoms_masses = lig_mass_h5,
                  numcores = 1, runmode = 'serial', temp = 310)

with open('ir_h5.csv', 'w', newline = '') as csvfil:
  writer = csv.writer(csvfil)
  writer.writerow(['freq_cm-1'] + ['IR_int_kmmol-1'])
  writer.writerow(zip(freq_h5.frequencies, freq_h5.IR_intensities))


freq_d5 = NumFreq(fragment = frag_prot, theory = qmmm,
                  hessatoms = lig_ind, hessatoms_masses = lig_mass_d5,
                  numcores = 1, runmode = 'serial', temp = 310)

with open('ir_d5.csv', 'w', newline = '') as csvfil:
  writer = csv.writer(csvfil)
  writer.writerow(['freq_cm-1'] + ['IR_int_kmmol-1'])
  writer.writerow(zip(freq_d5.frequencies, freq_d5.IR_intensities))